@@ -9,7 +9,7 @@ import distutils.dir_util
 import zbuilder.vm
 import zbuilder.cfg
 
-from zbuilder.helpers import getHosts, runPlaybook, fixKeys, runCmd, humanize_time
+from zbuilder.helpers import getHosts, getProviders, runPlaybook, fixKeys, runCmd, humanize_time
 from zbuilder.options import pass_state, common_options
 
 from click._bashcomplete import get_completion_script
@@ -204,6 +204,17 @@ def summary(state):
     click.echo(tabulate.tabulate(data, headers=["Provider", "Host", 'Parameters'], tablefmt="psql"))
 
 
+@cli.command()
+@pass_state
+def providers(state):
+    """Display providers info"""
+    click.echo('Checking providers state...')
+    cfg = zbuilder.cfg.load(touch=True)
+    data = getProviders(cfg, state)
+    headers = ['Name', 'Type', 'Check']
+    click.echo(tabulate.tabulate(data, headers=headers))
+
+
 @cli.group()
 def config():
     """Zbuilder configuration"""
