@@ -2,6 +2,7 @@ import os
 import re
 import time
 import click
+import requests
 import urllib.parse
 
 from proxmoxer import ProxmoxAPI
@@ -17,8 +18,10 @@ class vmProvider(object):
             password = self.cfg['password']
             url = self.cfg['url']
             verify = self.cfg.get('verify', True)
-
-            self.proxmox = ProxmoxAPI(url, user=self.username, password=password, verify_ssl=verify)
+            try:
+                self.proxmox = ProxmoxAPI(url, user=self.username, password=password, verify_ssl=verify)
+            except requests.exceptions.Timeout as e:
+                raise Exception(e.args[0].reason.args[1])
 
     def _waitTask(self, node, tid):
         results = None
@@ -62,7 +65,7 @@ class vmProvider(object):
 
     def build(self, hosts):
         ips = {}
-        vms = {i['name']: i for i in self.proxmox.cluster.resources.get(type='vm')}
+        templates = [i for i in self.proxmox.cluster.resources.get(type='vm') if i['template'] == 1]
 
         for h, v in self._getVMs(hosts).items():
             if v['status']:
@@ -72,13 +75,13 @@ class vmProvider(object):
                 click.echo("  - Creating host: {} ".format(h))
                 node = self.proxmox.nodes(v['node'])
                 nextid = self.proxmox.cluster.nextid.get()
-                if v['template'] in vms:
-                    template = vms[v['template']]
-                    if template['template'] != 1:
-                        click.echo("This is not a template: [{}]".format(v['template']))
-                        continue
-                else:
-                    click.echo("No such template: [{}]".format(v['template']))
+                template = None
+                for t in templates:
+                    if v['template'] == t['name'] and v['node'] == t['node']:
+                        template = t
+
+                if not template:
+                    click.echo("No such template: [{}] on node [{}]".format(v['template'], v['node']))
                     continue
 
                 # Clone the VM
@@ -114,7 +117,7 @@ class vmProvider(object):
                     sshkeys=urllib.parse.quote(sshkey, safe=''),
                     ciuser=v['ZBUILDER_SYSUSER']
                 )
-                for i, disk in enumerate(v['disks']):
+                for i, disk in enumerate(v.get('disks', [])):
                     args = {f"virtio{i+1}": disk}
                     taskid = node.qemu(nextid).config.set(**args)
 
@@ -128,10 +131,36 @@ class vmProvider(object):
         dnsUpdate(ips)
 
     def up(self, hosts):
-        pass
+        vms = {i['name']: i for i in self.proxmox.cluster.resources.get(type='vm')}
+        for h, v in self._getVMs(hosts).items():
+            if v['status']:
+                click.echo("  - Starting host: {} ".format(h))
+                node = self.proxmox.nodes(v['node'])
+                vm = vms[h]
+                if v['status'] == 'stopped':
+                    taskid = node(vm['id']).status.start().post()
+                    result = self._waitTask(node, taskid)
+                    if result != 'OK':
+                        click.echo('Failed: {}'.format(result))
+                        continue
+            else:
+                click.echo("  - Host does not exists [{}]".format(h))
 
     def halt(self, hosts):
-        pass
+        vms = {i['name']: i for i in self.proxmox.cluster.resources.get(type='vm')}
+        for h, v in self._getVMs(hosts).items():
+            if v['status']:
+                click.echo("  - Halting host: {} ".format(h))
+                node = self.proxmox.nodes(v['node'])
+                vm = vms[h]
+                if v['status'] == 'running':
+                    taskid = node(vm['id']).status.shutdown().post()
+                    result = self._waitTask(node, taskid)
+                    if result != 'OK':
+                        click.echo('Failed: {}'.format(result))
+                        continue
+            else:
+                click.echo("  - Host does not exists [{}]".format(h))
 
     def destroy(self, hosts):
         updateHosts = {}
@@ -178,5 +207,11 @@ class vmProvider(object):
                 ips[h] = None
         dnsRemove(ips)
 
+    def config(self):
+        return "url: {v[url]}, username: {v[username]}".format(v=self.cfg)
+
+    def status(self):
+        return 'PASS'
+
     def params(self, params):
-        return {k: params[k] for k in ['node', 'template', 'vcpu', 'memory', 'ipconfig']}
+        return {k: params.get(k, None) for k in ['node', 'template', 'vcpu', 'memory', 'ipconfig', 'disks']}
